@@ -11,12 +11,15 @@ from typing import Callable, ClassVar
 from openapi_client.api import accounts_api
 from openapi_client.api import campaigns_api
 from openapi_client.api import catalogs_api
+from openapi_client.api import coupons_api
 from openapi_client.api import data_privacy_api
 from openapi_client.api import events_api
 from openapi_client.api import flows_api
+from openapi_client.api import images_api
 from openapi_client.api import lists_api
 from openapi_client.api import metrics_api
 from openapi_client.api import profiles_api
+from openapi_client.api import reporting_api
 from openapi_client.api import segments_api
 from openapi_client.api import tags_api
 from openapi_client.api import templates_api
@@ -29,8 +32,10 @@ class KlaviyoAPI:
     max_delay: int = 60
     max_retries: int = 3
     test_host: str = ''
+    access_token: str = None
 
-    _REVISION = "2023-07-15"
+
+    _REVISION = "2024-02-15"
 
     _STATUS_CODE_CONNECTION_RESET_BY_PEER = 104
     _STATUS_CODE_TOO_MANY_REQUESTS = 429
@@ -48,12 +53,21 @@ class KlaviyoAPI:
 
     _CURSOR_SEARCH_TOKENS = ['page%5Bcursor%5D','page[cursor]']
 
+
     def __post_init__(self):
 
-        self.configuration = openapi_client.Configuration(
-            api_key={'Klaviyo-API-Key':f'Klaviyo-API-Key {self.api_key}'}
+        if self.access_token is not None:
+            
+            self.configuration = openapi_client.Configuration(
+                access_token=self.access_token
             )
 
+        elif self.api_key is not None:
+
+            self.configuration = openapi_client.Configuration(
+                api_key={'Klaviyo-API-Key':f'Klaviyo-API-Key {self.api_key}'}
+                )
+
         if self.test_host:
             self.configuration.host = self.test_host
 
@@ -61,16 +75,16 @@ class KlaviyoAPI:
 
         self.api_client.default_headers['revision'] = self._REVISION
         
+        if self.max_delay<= 0:
+            self.max_delay = .1
+        
         if self.max_retries <= 0:
-            self.max_wait = .1
-        else:
-            self.max_wait = self.max_delay/self.max_retries
-
-
+            self.max_retries = 1
+                
         self.retry_logic = tenacity.retry(
             reraise=True,
             retry=custom_retry.retry_if_qualifies(self._RETRY_CODES),
-            wait=tenacity.wait.wait_random_exponential(multiplier = 1, max = self.max_wait),
+            wait=tenacity.wait.wait_random_exponential(multiplier = 1, max = self.max_delay),
             stop=tenacity.stop.stop_after_attempt(self.max_retries)
         )
 
@@ -172,6 +186,29 @@ class KlaviyoAPI:
         self.Catalogs.update_catalog_variant=self._page_cursor_update(self.retry_logic(self.Catalogs.update_catalog_variant))
         
         
+        ## Adding Coupons to Client
+        self.Coupons=coupons_api.CouponsApi(self.api_client)
+        
+        ## Applying tenacity retry decorator to each endpoint in Coupons
+        self.Coupons.create_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.create_coupon))
+        self.Coupons.create_coupon_code=self._page_cursor_update(self.retry_logic(self.Coupons.create_coupon_code))
+        self.Coupons.delete_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.delete_coupon))
+        self.Coupons.delete_coupon_code=self._page_cursor_update(self.retry_logic(self.Coupons.delete_coupon_code))
+        self.Coupons.get_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon))
+        self.Coupons.get_coupon_code=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_code))
+        self.Coupons.get_coupon_code_bulk_create_job=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_code_bulk_create_job))
+        self.Coupons.get_coupon_code_bulk_create_jobs=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_code_bulk_create_jobs))
+        self.Coupons.get_coupon_code_relationships_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_code_relationships_coupon))
+        self.Coupons.get_coupon_codes=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_codes))
+        self.Coupons.get_coupon_codes_for_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_codes_for_coupon))
+        self.Coupons.get_coupon_for_coupon_code=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_for_coupon_code))
+        self.Coupons.get_coupon_relationships_coupon_codes=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupon_relationships_coupon_codes))
+        self.Coupons.get_coupons=self._page_cursor_update(self.retry_logic(self.Coupons.get_coupons))
+        self.Coupons.spawn_coupon_code_bulk_create_job=self._page_cursor_update(self.retry_logic(self.Coupons.spawn_coupon_code_bulk_create_job))
+        self.Coupons.update_coupon=self._page_cursor_update(self.retry_logic(self.Coupons.update_coupon))
+        self.Coupons.update_coupon_code=self._page_cursor_update(self.retry_logic(self.Coupons.update_coupon_code))
+        
+        
         ## Adding Data_Privacy to Client
         self.Data_Privacy=data_privacy_api.DataPrivacyApi(self.api_client)
         
@@ -206,6 +243,8 @@ class KlaviyoAPI:
         self.Flows.get_flow_message=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_message))
         self.Flows.get_flow_message_action=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_message_action))
         self.Flows.get_flow_message_relationships_action=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_message_relationships_action))
+        self.Flows.get_flow_message_relationships_template=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_message_relationships_template))
+        self.Flows.get_flow_message_template=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_message_template))
         self.Flows.get_flow_relationships_flow_actions=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_relationships_flow_actions))
         self.Flows.get_flow_relationships_tags=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_relationships_tags))
         self.Flows.get_flow_tags=self._page_cursor_update(self.retry_logic(self.Flows.get_flow_tags))
@@ -213,6 +252,17 @@ class KlaviyoAPI:
         self.Flows.update_flow=self._page_cursor_update(self.retry_logic(self.Flows.update_flow))
         
         
+        ## Adding Images to Client
+        self.Images=images_api.ImagesApi(self.api_client)
+        
+        ## Applying tenacity retry decorator to each endpoint in Images
+        self.Images.get_image=self._page_cursor_update(self.retry_logic(self.Images.get_image))
+        self.Images.get_images=self._page_cursor_update(self.retry_logic(self.Images.get_images))
+        self.Images.update_image=self._page_cursor_update(self.retry_logic(self.Images.update_image))
+        self.Images.upload_image_from_file=self._page_cursor_update(self.retry_logic(self.Images.upload_image_from_file))
+        self.Images.upload_image_from_url=self._page_cursor_update(self.retry_logic(self.Images.upload_image_from_url))
+        
+        
         ## Adding Lists to Client
         self.Lists=lists_api.ListsApi(self.api_client)
         
@@ -243,13 +293,24 @@ class KlaviyoAPI:
         self.Profiles=profiles_api.ProfilesApi(self.api_client)
         
         ## Applying tenacity retry decorator to each endpoint in Profiles
+        self.Profiles.create_or_update_profile=self._page_cursor_update(self.retry_logic(self.Profiles.create_or_update_profile))
         self.Profiles.create_profile=self._page_cursor_update(self.retry_logic(self.Profiles.create_profile))
+        self.Profiles.create_push_token=self._page_cursor_update(self.retry_logic(self.Profiles.create_push_token))
+        self.Profiles.get_bulk_profile_import_job=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job))
+        self.Profiles.get_bulk_profile_import_job_import_errors=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job_import_errors))
+        self.Profiles.get_bulk_profile_import_job_lists=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job_lists))
+        self.Profiles.get_bulk_profile_import_job_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job_profiles))
+        self.Profiles.get_bulk_profile_import_job_relationships_lists=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job_relationships_lists))
+        self.Profiles.get_bulk_profile_import_job_relationships_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_job_relationships_profiles))
+        self.Profiles.get_bulk_profile_import_jobs=self._page_cursor_update(self.retry_logic(self.Profiles.get_bulk_profile_import_jobs))
         self.Profiles.get_profile=self._page_cursor_update(self.retry_logic(self.Profiles.get_profile))
         self.Profiles.get_profile_lists=self._page_cursor_update(self.retry_logic(self.Profiles.get_profile_lists))
         self.Profiles.get_profile_relationships_lists=self._page_cursor_update(self.retry_logic(self.Profiles.get_profile_relationships_lists))
         self.Profiles.get_profile_relationships_segments=self._page_cursor_update(self.retry_logic(self.Profiles.get_profile_relationships_segments))
         self.Profiles.get_profile_segments=self._page_cursor_update(self.retry_logic(self.Profiles.get_profile_segments))
         self.Profiles.get_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.get_profiles))
+        self.Profiles.merge_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.merge_profiles))
+        self.Profiles.spawn_bulk_profile_import_job=self._page_cursor_update(self.retry_logic(self.Profiles.spawn_bulk_profile_import_job))
         self.Profiles.subscribe_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.subscribe_profiles))
         self.Profiles.suppress_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.suppress_profiles))
         self.Profiles.unsubscribe_profiles=self._page_cursor_update(self.retry_logic(self.Profiles.unsubscribe_profiles))
@@ -257,6 +318,15 @@ class KlaviyoAPI:
         self.Profiles.update_profile=self._page_cursor_update(self.retry_logic(self.Profiles.update_profile))
         
         
+        ## Adding Reporting to Client
+        self.Reporting=reporting_api.ReportingApi(self.api_client)
+        
+        ## Applying tenacity retry decorator to each endpoint in Reporting
+        self.Reporting.query_campaign_values=self._page_cursor_update(self.retry_logic(self.Reporting.query_campaign_values))
+        self.Reporting.query_flow_series=self._page_cursor_update(self.retry_logic(self.Reporting.query_flow_series))
+        self.Reporting.query_flow_values=self._page_cursor_update(self.retry_logic(self.Reporting.query_flow_values))
+        
+        
         ## Adding Segments to Client
         self.Segments=segments_api.SegmentsApi(self.api_client)
         
